// wedding badges - recieve
// ------------------------
// authors :
// adelle lin
// sophi kravitz
// matt pinner
//

// low power utility functions
// note: hacked ATMEGA328PB into header file for our dev board
#include <LowPower.h> //get library from: https://github.com/lowpowerlab/lowpower
//writeup here: http://www.rocketscream.com/blog/2011/07/04/lightweight-low-power-arduino-library/

/************ Radio Setup ***************/
// communication util for rfm69
#include <RH_RF69.h>
#define RF69_FREQ  915  // 915hz
#define RFM69_INT    2  //radio pins
#define RFM69_CS    10
#define RFM69_RST   24

// encryption key has to be the same as the one in the server
uint8_t key[] = { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
                  0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08
                };
// Singleton instance of the radio driver
RH_RF69 rf69(RFM69_CS, RFM69_INT);

/************ Animation Setup ***************/
#include <SoftPWM.h>

// led state, mode, pins, initialization, and animation settings
bool awake = true;  // should this stay sleeping

// led count and pinouts
//const int NUM_LEDS = 4;
//int ledPins[] =  {3, 5, 6, 9};  // these are the pwms
const int NUM_LEDS = 8;
int ledPins[] =  {3, 4, 23, 5, 6, 7, 8, 9};

// timers for animation cycles
#include <elapsedMillis.h>
elapsedMillis animationElapsed;
elapsedMillis waitSleepElapsed;
int animationDuration = 200;  // duration of each animation cycle in ms
int waitSleepDuration = 3000;

int animationMode = 1; // choosen animation mode
int animationValue = 0;  // used for fade up and then down?

// state to drive the animation per led
int selected[NUM_LEDS];

// global brightness max
int brightness = 255;

void setup()
{
  Serial.begin(115200);
  while (!Serial) {
    delay(1);  // wait until serial console is open, remove if not tethered to computer
  }

  // output last filename uploaded and date
  Serial.print("uploaded: '");
  Serial.print(__FILE__);
  Serial.print("' on ");
  Serial.println(__DATE__);
  
  setupLeds();  // init leds
  setupRadio(); // init radio
  
}

void loop() {
  // receive message 
  // set the mode for animation
  receive_message();
  check_if_should_sleep();

  // 
  if (awake == true) {
    
    // double animation duration to allow fade up and down
    if (animationElapsed > 2 * animationDuration) {
      //      start_0();
      //Serial.println("starting animation");
      start_chase_0();
      animationElapsed = 0;
    }
    else if (animationElapsed > animationDuration) {
      animationValue = 2 * animationDuration - animationElapsed;
    }
    else {
      animationValue = animationElapsed;
    }
    //    calc_percent_remaining();

    if (animationMode == 0) {
      animateDigital();
    }
    if (animationMode == 1) {
      animateFade();
    }
  }

  if (awake == false) {
    //LowPower.powerSave(SLEEP_15MS, ADC_ON, BOD_OFF, TIMER2_ON);
    LowPower.powerDown(SLEEP_2S, ADC_ON, BOD_OFF);
  }
  delay(5);
}

void Blink(byte PIN, byte DELAY_MS, byte loops) {
  for (byte i = 0; i < loops; i++)  {
    digitalWrite(PIN, HIGH);
    delay(DELAY_MS);
    digitalWrite(PIN, LOW);
    delay(DELAY_MS);
  }
}


void start_0() {
  for (int i = 0; i < NUM_LEDS; i++) {
    if (i % 2 == 0) {
      selected[i] = 1;
    } else {
      selected[i] = 0;
    }
  }
}

void start_chase_0() {
  // at every loop, flip the value of the next index to one
  // and self to zero
  for (int i = NUM_LEDS - 1; i >= 0 ; i--) {
    // wrap around condition, last led to 0, first led to 1
    if (selected[NUM_LEDS - 1] == 1) {
      selected[NUM_LEDS - 1] = 0;
      selected[0] = 1;
      break;
    }
    else if (selected[i] == 1) {
      selected[i] = 0;
      selected[i + 1] = 1;
      break;
    }
  }
}

void animateFade() {
  float percentComplete = (((float)animationValue) / (float)animationDuration);
  // check each led
  for (int i = 0; i < NUM_LEDS; i++) {
    float fade = percentComplete; // fade up
    if (selected[i] == 0) {
      fade = 0.0;
    }
    // set the one on chase to fade up
    setBrightness(i, fade * brightness); //*random(1,3)); <- gives a sparkle
  }
}

void animateDigital() {
  for (int i = 0; i < NUM_LEDS; i++) {
    setLed(i, selected[i] == 1);
  }
}

void receive_message() {
  if (rf69.available()) {
    // Should be a message for us now
    uint8_t buf[RH_RF69_MAX_MESSAGE_LEN];
    uint8_t len = sizeof(buf);
    if (rf69.recv(buf, &len)) {
      if (!len) return;
      awake = true;
      int receivedMessage = atoi((char*)buf);
      handleModeOnReceive(receivedMessage);
      Serial.println(animationMode);
      waitSleepElapsed = 0;
      receive_debug(len, buf);
      buf[len] = 0;
      if (strstr((char *)buf, "Hello World")) {
        // Send a reply!
        send_ack();
      }
    } else {
       receive_fail();
    }
  }
}

void handleModeOnReceive(int receivedMessage){
  if (animationMode != receivedMessage){
    for (int i = 0; i < NUM_LEDS; i++) {
      setBrightness(i, 0);
      selected[i] = 0;
    }
    selected[0] = 1;
    animationMode = receivedMessage;
  }
}

void receive_fail() {
  Serial.println("Receive failed");
  digitalWrite(23, HIGH);
  digitalWrite(5, HIGH);
  digitalWrite(3, HIGH);
  digitalWrite(4, HIGH);
  digitalWrite(6, HIGH);
  delay(30);
  digitalWrite(6, LOW);
  delay(30);
}

void send_ack() {
  uint8_t data[] = "And hello back to you";
  rf69.send(data, sizeof(data));
  rf69.waitPacketSent();
  Serial.println("Sent a reply");
  digitalWrite(23, HIGH);
  digitalWrite(6, HIGH);
  digitalWrite(3, HIGH);
  digitalWrite(4, HIGH);
  // reply leds
  digitalWrite(5, HIGH);
  delay(50);
  digitalWrite(5, LOW);
  delay(50);
  digitalWrite(5, HIGH);
  delay(50);
  digitalWrite(5, LOW);
  delay(50);
}

void receive_debug(uint8_t len, uint8_t *buf) {
  Serial.print("Received [");
  Serial.print(len);
  Serial.print("]: ");
  Serial.println((char*)buf);
  Serial.print("RSSI: ");
  Serial.println(rf69.lastRssi(), DEC);
  digitalWrite(6, HIGH);
  digitalWrite(5, HIGH);
  digitalWrite(3, HIGH);
  digitalWrite(4, HIGH);
  receive_leds_on();
}

void check_if_should_sleep(){
    if (waitSleepElapsed > waitSleepDuration) {
    awake = false;
    setAllOff();
  }
}

void receive_leds_on() {
  digitalWrite(23, HIGH);
  delay(100);
  digitalWrite(23, LOW);
  delay(100);
}

void setLed(int i, bool isOn) {
  digitalWrite(ledPins[i], !isOn);
}

// fade is flaot scaled between 0.0 and brightness
void setBrightness(int i, float fade) {
  //analogWrite(ledPins[i], (int)(brightness - fade));
  SoftPWMSet(ledPins[i], fade);

}

void setAllOff() {
  for (int i = 0; i < NUM_LEDS; i++)
    setLed(i, false);
}

void setAllOn() {
  for (int i = 0; i < NUM_LEDS; i++)
    setLed(i, true);
}


// init led pins and initialize selected state
void setupLeds() {
  SoftPWMBegin();
  for (int i = 0; i < NUM_LEDS; i++) {
    SoftPWMSet(ledPins[i], 0);
    selected[i] = 0;
  }
  SoftPWMSetFadeTime(ALL, 50, 400);
  SoftPWMSetPolarity(ALL, SOFTPWM_INVERTED);

  setAllOff();
  selected[0] = 1; 
}


void setupRadio() {

  // set radio pin modes
  pinMode(RFM69_RST, OUTPUT);
  digitalWrite(RFM69_RST, LOW);
  Serial.println("wedding badge recieve!");
  Serial.println();

  // manual reset
  digitalWrite(RFM69_RST, HIGH);
  delay(10);
  digitalWrite(RFM69_RST, LOW);
  delay(10);

  if (!rf69.init()) {
    Serial.println("RFM69 radio init failed");
    digitalWrite(23, HIGH);
    digitalWrite(6, HIGH);
    digitalWrite(5, HIGH);
    digitalWrite(4, HIGH);
    digitalWrite(3, HIGH);
    delay(30);
    digitalWrite(3, LOW);
    delay(30);
    while (1);
  }
  Serial.println("RFM69 radio init OK!");

  digitalWrite(23, HIGH);
  digitalWrite(6, HIGH);
  digitalWrite(5, HIGH);
  digitalWrite(3, HIGH);
  digitalWrite(4, HIGH);
  delay(50);
  digitalWrite(4, LOW);
  delay(50);
  // Defaults after init are 434.0MHz, modulation GFSK_Rb250Fd250, +13dbM (for low power module)
  // No encryption
  if (!rf69.setFrequency(RF69_FREQ)) {
    Serial.println("setFrequency failed");
  }

  // If you are using a high power RF69 eg RFM69HW, you *must* set a Tx power with the
  // ishighpowermodule flag set like this:
  rf69.setTxPower(20, true);  // range from 14-20 for power, 2nd arg must be true for 69HCW
  rf69.setEncryptionKey(key);
  Serial.print("RFM69 radio @");  Serial.print((int)RF69_FREQ);  Serial.println(" MHz");
}
