// wedding badges - transmit
// ------------------------
// authors :
// adelle lin
// sophi kravitz
// matt pinner
//

// rf69 demo tx rx.pde
// -*- mode: C++ -*-
// Example sketch showing how to create a simple messageing client
// with the RH_RF69 class. RH_RF69 class does not provide for addressing or
// reliability, so you should only use RH_RF69  if you do not need the higher
// level messaging abilities.
// It is designed to work with the other example rf69_server.
// Demonstrates the use of AES encryption, setting the frequency and modem
// configuration

//    digitalWrite(0)         //--> D0
//    digitalWrite(1)         //--> D1
//    digitalWrite(2)         //--> D2
//    digitalWrite(3,LOW);    //--> D3
//    digitalWrite(4,LOW);    //--> D4
//    digitalWrite(5,LOW);    //--> D5
//    digitalWrite(6,LOW);    //--> D6
//    digitalWrite(7,LOW);    //--> D7
//    digitalWrite(8,HIGH);   //--> B0
//    digitalWrite(9,HIGH);   //--> B1
//    digitalWrite(10,HIGH);  //--> B2 SSO
//    digitalWrite(11,HIGH);  //--> B3 MOSI
//    digitalWrite(12,HIGH);  //--> B4 MISO
//    digitalWrite(13,HIGH);  //--> B5 SCK
//    digitalWrite(14,LOW);   //--> C0
//    digitalWrite(15,LOW);   //--> C1
//    digitalWrite(16,LOW);   //--> C2
//    digitalWrite(17,LOW);   //--> C3
//    digitalWrite(18,LOW);   //--> XX
//    digitalWrite(19,LOW);   //--> XX
//    digitalWrite(20,LOW);   //--> XX
//    digitalWrite(21,LOW);   //--> XX
//    digitalWrite(22,LOW);   //--> XX
//    digitalWrite(23,LOW);   //--> E0
//    digitalWrite(24,LOW);   //--> E1
//    pinMode(A2, INPUT_PULLUP); //-->25 (ALSO C2)

#include <SPI.h>
#include <RH_RF69.h>

/************ Radio Setup ***************/
// Change to 434.0 or other frequency, must match RX's freq!
#define RF69_FREQ 915
#define RFM69_INT   2  // 
#define RFM69_CS   10  //
#define RFM69_RST  24  // "A"
int hours = 18;
int minutes = 56;
int seconds = 0;
int days = 1;
int months = 2;
int years = 2020;
unsigned len;
// Singleton instance of the radio driver
RH_RF69 rf69(RFM69_CS, RFM69_INT);

// The encryption key has to be the same as the one in the server
uint8_t key[] = { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
                  0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08
                };

/************ Controls Setup ***************/
#include <elapsedMillis.h>
elapsedMillis modeElapsed;
int controlDuration = 5000;
int sleepDuration = 15000;
bool controlMode = false;
int mode_index = 0;
int send_index = 66;
int duration = 0;

// led count and pinouts
const int NUM_LEDS = 4;
int ledPins[] = {3, 5, 6, 9}; // pwm
//int ledPins[] =  {3, 4, 23, 5, 6, 7, 8, 9};

int16_t packetnum = 0;  // packet counter, we increment per xmission, 16 bit int

// state to drive the animation per led
int selected[NUM_LEDS];

// modes
const int NUM_MODES = 2;
int controlModes[NUM_MODES] = {0, 1};

void setup()
{
  //  setTime(hours, minutes, seconds, days, months, years);
  Serial.begin(115200);
  while (!Serial) {
    delay(1);  // wait until serial console is open, remove if not tethered to computer
  }

  // output last filename uploaded and date
  Serial.print("uploaded: '");
  Serial.print(__FILE__);
  Serial.print("' on ");
  Serial.println(__DATE__);
  pinMode(A2, INPUT_PULLUP); // CHECK IF INTERNAL PULLUP
  setupLeds();
  setupRadio();
}

int count = 0;
void loop() {
  
  delay(900);  // Wait 1 second between transmits, could also 'sleep' here!
  int sensorValue = analogRead(A2); // int
  //  Serial.println(sensorValue);
  //  Serial.println(count);
  if (controlMode == true) {
    duration = controlDuration;
  } else {
    duration = sleepDuration;
  }
  if (modeElapsed > duration) {
    if (controlMode == false) {
      // was in sleep mode, send control message
      modeElapsed = 0;
      controlMode = true;
      mode_index = (mode_index+1) % NUM_MODES;
      send_index = mode_index;
    } else {
      // go to sleep mode and don't send messages
      modeElapsed = 0;
      controlMode = false;
      send_index = 66;
    }
  }
  // if want to send messages during sleep cycle for debug
  if (send_index != 66) {
  send_message(send_index);
  }
}

void Blink(byte PIN, byte DELAY_MS, byte loops) {
  for (byte i = 0; i < loops; i++)  {
    digitalWrite(PIN, HIGH);
    delay(DELAY_MS);
    digitalWrite(PIN, LOW);
    delay(DELAY_MS);
  }
}

void send_message(int mode_index) {
  String b;
  char c[4];
  //  b = (String)sensorValue;
//  b = (String)count++ + "   ";
  b = (String)mode_index + "  ";
  b.toCharArray(c, 4);
//  b = "Hello World";
//  b.toCharArray(c, 12);
  rf69.send((uint8_t *)c, strlen(c)); // original code byte, strlen (returns long int)
  Serial.print("Sending: "); Serial.print(c); //Serial.println(strlen(c));
  // rf69.send((uint8_t *)radiopacket, strlen(radiopacket)); // original code byte, strlen (returns long int)
  // param[in] data Array of data to be sent
  // param[in] len Number of bytes of data to send (> 0)
  // return true if the message length was valid and it was correctly queued for transmit
  rf69.waitPacketSent();
  digitalWrite(6, HIGH);
  digitalWrite(5, HIGH);
  digitalWrite(3, HIGH);
  digitalWrite(4, HIGH);
  digitalWrite(23, LOW);
  delay(30);
  digitalWrite(23, HIGH);
  delay(30);

  check_for_reply();
}

void check_for_reply() {
  // Now wait for a reply
  uint8_t buf[RH_RF69_MAX_MESSAGE_LEN];
  uint8_t len = sizeof(buf);

  if (rf69.waitAvailableTimeout(500))  {
    // Should be a reply message for us now
    get_reply_message(len, buf);
  } else {
  no_reply();
  }
}

void get_reply_message(uint8_t len, uint8_t *buf) {
  if (rf69.recv(buf, &len)) {
    Serial.print("Got a reply: ");
    Serial.println((char*)buf);
    digitalWrite(23, HIGH);
    digitalWrite(6, HIGH);
    digitalWrite(3, HIGH);
    digitalWrite(4, HIGH);
    digitalWrite(5, LOW);
    delay(50);
    digitalWrite(5, HIGH);
    delay(50);
    digitalWrite(5, LOW);
    delay(50);
    digitalWrite(5, HIGH);
    delay(50);
    digitalWrite(5, LOW);
    delay(50);
  } else {
    Serial.println("Receive failed");
  } 
}

void no_reply() {
  Serial.println("No reply, is another RFM69 listening?");
  digitalWrite(23, HIGH);
  digitalWrite(5, HIGH);
  digitalWrite(3, HIGH);
  digitalWrite(4, HIGH);
  digitalWrite(6, LOW);
  delay(30);
  digitalWrite(6, HIGH);
  delay(30);
  digitalWrite(6, LOW);
  delay(30);
  digitalWrite(6, HIGH);
}

void setAllOff() {
  for (int i = 0; i < NUM_LEDS; i++)
    setLed(i, false);
}

void setAllOn() {
  for (int i = 0; i < NUM_LEDS; i++)
    setLed(i, true);
}

void setLed(int i, bool isOn) {
  // leds light up when grounded
  digitalWrite(ledPins[i], !isOn);
}

void setupLeds() {
  // at start, set all led state to 0
  for (int i = 0; i < NUM_LEDS; i++) {
    pinMode(ledPins[i], OUTPUT);
    selected[i] = 0;
  }
  // turn all LEDs off
  setAllOff();
}

void setupRadio() {
  pinMode(RFM69_RST, OUTPUT);
  digitalWrite(RFM69_RST, LOW);

  Serial.println("Feather RFM69 TX Test!");
  Serial.println();

  // manual reset
  digitalWrite(RFM69_RST, HIGH);
  delay(10);
  digitalWrite(RFM69_RST, LOW);
  delay(10);

  if (!rf69.init()) {
    Serial.println("RFM69 radio init failed");
    digitalWrite(23, HIGH);
    digitalWrite(6, HIGH);
    digitalWrite(5, HIGH);
    digitalWrite(4, HIGH);
    digitalWrite(3, HIGH);
    delay(30);
    digitalWrite(3, LOW);
    delay(30);
    while (1);
  }
  Serial.println("RFM69 radio init OK!");
  digitalWrite(23, HIGH);
  digitalWrite(6, HIGH);
  digitalWrite(5, HIGH);
  digitalWrite(3, HIGH);
  digitalWrite(4, HIGH);
  delay(50);
  digitalWrite(4, LOW);
  delay(50);
  // Defaults after init are 434.0MHz, modulation GFSK_Rb250Fd250, +13dbM (for low power module)
  // No encryption
  if (!rf69.setFrequency(RF69_FREQ)) {
    Serial.println("setFrequency failed");
  }

  // If you are using a high power RF69 eg RFM69HW, you *must* set a Tx power with the
  // ishighpowermodule flag set like this:
  rf69.setTxPower(20, true);  // range from 14-20 for power, 2nd arg must be true for 69HCW
  rf69.setEncryptionKey(key);

  Serial.print("RFM69 radio @");  Serial.print((int)RF69_FREQ);  Serial.println(" MHz");

}
